# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_room.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import attestation_pb2 as attestation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x64\x61ta_room.proto\x12\tdata_room\x1a\x11\x61ttestation.proto\"\xb9\x02\n\x08\x44\x61taRoom\x12\n\n\x02id\x18\x01 \x01(\t\x12,\n\x0c\x63omputeNodes\x18\x02 \x03(\x0b\x32\x16.data_room.ComputeNode\x12H\n\x19\x61ttestationSpecifications\x18\x03 \x03(\x0b\x32%.attestation.AttestationSpecification\x12\x32\n\x0fuserPermissions\x18\x04 \x03(\x0b\x32\x19.data_room.UserPermission\x12>\n\x15\x61uthenticationMethods\x18\x05 \x03(\x0b\x32\x1f.data_room.AuthenticationMethod\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x07 \x01(\t\x12\x12\n\nownerEmail\x18\x08 \x01(\t\"\x83\x01\n\x0b\x43omputeNode\x12\x10\n\x08nodeName\x18\x01 \x01(\t\x12*\n\x04leaf\x18\x02 \x01(\x0b\x32\x1a.data_room.ComputeNodeLeafH\x00\x12.\n\x06\x62ranch\x18\x03 \x01(\x0b\x32\x1c.data_room.ComputeNodeBranchH\x00\x42\x06\n\x04node\"%\n\x0f\x43omputeNodeLeaf\x12\x12\n\nisRequired\x18\x01 \x01(\x08\"\x94\x01\n\x11\x43omputeNodeBranch\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\x0c\x12\x14\n\x0c\x64\x65pendencies\x18\x02 \x03(\t\x12%\n\x1d\x61ttestationSpecificationIndex\x18\x03 \x01(\r\x12\x32\n\x0coutputFormat\x18\x04 \x01(\x0e\x32\x1c.data_room.ComputeNodeFormat\"n\n\x0eUserPermission\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12!\n\x19\x61uthenticationMethodIndex\x18\x02 \x01(\r\x12*\n\x0bpermissions\x18\x03 \x03(\x0b\x32\x15.data_room.Permission\"U\n\x14\x41uthenticationMethod\x12.\n\ntrustedPki\x18\x01 \x01(\x0b\x32\x15.data_room.TrustedPkiH\x00\x88\x01\x01\x42\r\n\x0b_trustedPki\"(\n\nTrustedPki\x12\x1a\n\x12rootCertificatePem\x18\x01 \x01(\x0c\"\x80\x05\n\nPermission\x12G\n\x18\x65xecuteComputePermission\x18\x01 \x01(\x0b\x32#.data_room.ExecuteComputePermissionH\x00\x12;\n\x12leafCrudPermission\x18\x02 \x01(\x0b\x32\x1d.data_room.LeafCrudPermissionH\x00\x12K\n\x1aretrieveDataRoomPermission\x18\x03 \x01(\x0b\x32%.data_room.RetrieveDataRoomPermissionH\x00\x12K\n\x1aretrieveAuditLogPermission\x18\x04 \x01(\x0b\x32%.data_room.RetrieveAuditLogPermissionH\x00\x12W\n retrieveDataRoomStatusPermission\x18\x05 \x01(\x0b\x32+.data_room.RetrieveDataRoomStatusPermissionH\x00\x12S\n\x1eupdateDataRoomStatusPermission\x18\x06 \x01(\x0b\x32).data_room.UpdateDataRoomStatusPermissionH\x00\x12]\n#retrievePublishedDatasetsPermission\x18\x07 \x01(\x0b\x32..data_room.RetrievePublishedDatasetsPermissionH\x00\x12\x37\n\x10\x64ryRunPermission\x18\x08 \x01(\x0b\x32\x1b.data_room.DryRunPermissionH\x00\x42\x0c\n\npermission\"3\n\x18\x45xecuteComputePermission\x12\x17\n\x0f\x63omputeNodeName\x18\x01 \x01(\t\"*\n\x12LeafCrudPermission\x12\x14\n\x0cleafNodeName\x18\x01 \x01(\t\"\x1c\n\x1aRetrieveDataRoomPermission\"\x1c\n\x1aRetrieveAuditLogPermission\"\"\n RetrieveDataRoomStatusPermission\" \n\x1eUpdateDataRoomStatusPermission\"%\n#RetrievePublishedDatasetsPermission\"\x12\n\x10\x44ryRunPermission*%\n\x11\x43omputeNodeFormat\x12\x07\n\x03RAW\x10\x00\x12\x07\n\x03ZIP\x10\x01\x62\x06proto3')

_COMPUTENODEFORMAT = DESCRIPTOR.enum_types_by_name['ComputeNodeFormat']
ComputeNodeFormat = enum_type_wrapper.EnumTypeWrapper(_COMPUTENODEFORMAT)
RAW = 0
ZIP = 1


_DATAROOM = DESCRIPTOR.message_types_by_name['DataRoom']
_COMPUTENODE = DESCRIPTOR.message_types_by_name['ComputeNode']
_COMPUTENODELEAF = DESCRIPTOR.message_types_by_name['ComputeNodeLeaf']
_COMPUTENODEBRANCH = DESCRIPTOR.message_types_by_name['ComputeNodeBranch']
_USERPERMISSION = DESCRIPTOR.message_types_by_name['UserPermission']
_AUTHENTICATIONMETHOD = DESCRIPTOR.message_types_by_name['AuthenticationMethod']
_TRUSTEDPKI = DESCRIPTOR.message_types_by_name['TrustedPki']
_PERMISSION = DESCRIPTOR.message_types_by_name['Permission']
_EXECUTECOMPUTEPERMISSION = DESCRIPTOR.message_types_by_name['ExecuteComputePermission']
_LEAFCRUDPERMISSION = DESCRIPTOR.message_types_by_name['LeafCrudPermission']
_RETRIEVEDATAROOMPERMISSION = DESCRIPTOR.message_types_by_name['RetrieveDataRoomPermission']
_RETRIEVEAUDITLOGPERMISSION = DESCRIPTOR.message_types_by_name['RetrieveAuditLogPermission']
_RETRIEVEDATAROOMSTATUSPERMISSION = DESCRIPTOR.message_types_by_name['RetrieveDataRoomStatusPermission']
_UPDATEDATAROOMSTATUSPERMISSION = DESCRIPTOR.message_types_by_name['UpdateDataRoomStatusPermission']
_RETRIEVEPUBLISHEDDATASETSPERMISSION = DESCRIPTOR.message_types_by_name['RetrievePublishedDatasetsPermission']
_DRYRUNPERMISSION = DESCRIPTOR.message_types_by_name['DryRunPermission']
DataRoom = _reflection.GeneratedProtocolMessageType('DataRoom', (_message.Message,), {
  'DESCRIPTOR' : _DATAROOM,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.DataRoom)
  })
_sym_db.RegisterMessage(DataRoom)

ComputeNode = _reflection.GeneratedProtocolMessageType('ComputeNode', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTENODE,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.ComputeNode)
  })
_sym_db.RegisterMessage(ComputeNode)

ComputeNodeLeaf = _reflection.GeneratedProtocolMessageType('ComputeNodeLeaf', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTENODELEAF,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.ComputeNodeLeaf)
  })
_sym_db.RegisterMessage(ComputeNodeLeaf)

ComputeNodeBranch = _reflection.GeneratedProtocolMessageType('ComputeNodeBranch', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTENODEBRANCH,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.ComputeNodeBranch)
  })
_sym_db.RegisterMessage(ComputeNodeBranch)

UserPermission = _reflection.GeneratedProtocolMessageType('UserPermission', (_message.Message,), {
  'DESCRIPTOR' : _USERPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.UserPermission)
  })
_sym_db.RegisterMessage(UserPermission)

AuthenticationMethod = _reflection.GeneratedProtocolMessageType('AuthenticationMethod', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATIONMETHOD,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.AuthenticationMethod)
  })
_sym_db.RegisterMessage(AuthenticationMethod)

TrustedPki = _reflection.GeneratedProtocolMessageType('TrustedPki', (_message.Message,), {
  'DESCRIPTOR' : _TRUSTEDPKI,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.TrustedPki)
  })
_sym_db.RegisterMessage(TrustedPki)

Permission = _reflection.GeneratedProtocolMessageType('Permission', (_message.Message,), {
  'DESCRIPTOR' : _PERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.Permission)
  })
_sym_db.RegisterMessage(Permission)

ExecuteComputePermission = _reflection.GeneratedProtocolMessageType('ExecuteComputePermission', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTECOMPUTEPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.ExecuteComputePermission)
  })
_sym_db.RegisterMessage(ExecuteComputePermission)

LeafCrudPermission = _reflection.GeneratedProtocolMessageType('LeafCrudPermission', (_message.Message,), {
  'DESCRIPTOR' : _LEAFCRUDPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.LeafCrudPermission)
  })
_sym_db.RegisterMessage(LeafCrudPermission)

RetrieveDataRoomPermission = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomPermission', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.RetrieveDataRoomPermission)
  })
_sym_db.RegisterMessage(RetrieveDataRoomPermission)

RetrieveAuditLogPermission = _reflection.GeneratedProtocolMessageType('RetrieveAuditLogPermission', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEAUDITLOGPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.RetrieveAuditLogPermission)
  })
_sym_db.RegisterMessage(RetrieveAuditLogPermission)

RetrieveDataRoomStatusPermission = _reflection.GeneratedProtocolMessageType('RetrieveDataRoomStatusPermission', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEDATAROOMSTATUSPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.RetrieveDataRoomStatusPermission)
  })
_sym_db.RegisterMessage(RetrieveDataRoomStatusPermission)

UpdateDataRoomStatusPermission = _reflection.GeneratedProtocolMessageType('UpdateDataRoomStatusPermission', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATAROOMSTATUSPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.UpdateDataRoomStatusPermission)
  })
_sym_db.RegisterMessage(UpdateDataRoomStatusPermission)

RetrievePublishedDatasetsPermission = _reflection.GeneratedProtocolMessageType('RetrievePublishedDatasetsPermission', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEPUBLISHEDDATASETSPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.RetrievePublishedDatasetsPermission)
  })
_sym_db.RegisterMessage(RetrievePublishedDatasetsPermission)

DryRunPermission = _reflection.GeneratedProtocolMessageType('DryRunPermission', (_message.Message,), {
  'DESCRIPTOR' : _DRYRUNPERMISSION,
  '__module__' : 'data_room_pb2'
  # @@protoc_insertion_point(class_scope:data_room.DryRunPermission)
  })
_sym_db.RegisterMessage(DryRunPermission)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMPUTENODEFORMAT._serialized_start=1859
  _COMPUTENODEFORMAT._serialized_end=1896
  _DATAROOM._serialized_start=50
  _DATAROOM._serialized_end=363
  _COMPUTENODE._serialized_start=366
  _COMPUTENODE._serialized_end=497
  _COMPUTENODELEAF._serialized_start=499
  _COMPUTENODELEAF._serialized_end=536
  _COMPUTENODEBRANCH._serialized_start=539
  _COMPUTENODEBRANCH._serialized_end=687
  _USERPERMISSION._serialized_start=689
  _USERPERMISSION._serialized_end=799
  _AUTHENTICATIONMETHOD._serialized_start=801
  _AUTHENTICATIONMETHOD._serialized_end=886
  _TRUSTEDPKI._serialized_start=888
  _TRUSTEDPKI._serialized_end=928
  _PERMISSION._serialized_start=931
  _PERMISSION._serialized_end=1571
  _EXECUTECOMPUTEPERMISSION._serialized_start=1573
  _EXECUTECOMPUTEPERMISSION._serialized_end=1624
  _LEAFCRUDPERMISSION._serialized_start=1626
  _LEAFCRUDPERMISSION._serialized_end=1668
  _RETRIEVEDATAROOMPERMISSION._serialized_start=1670
  _RETRIEVEDATAROOMPERMISSION._serialized_end=1698
  _RETRIEVEAUDITLOGPERMISSION._serialized_start=1700
  _RETRIEVEAUDITLOGPERMISSION._serialized_end=1728
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_start=1730
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_end=1764
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_start=1766
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_end=1798
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_start=1800
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_end=1837
  _DRYRUNPERMISSION._serialized_start=1839
  _DRYRUNPERMISSION._serialized_end=1857
# @@protoc_insertion_point(module_scope)



import chily
import os
from hashlib import sha256
from .proto import serialize_length_delimited
from .proto import ChunkHeader, EncryptionHeader, VersionHeader
from typing import BinaryIO, Iterator, Tuple, Optional
from io import TextIOWrapper

__all__ = ["Key"]

KEY_LEN = 32

class Key():
    """
    This class wraps the key material that is used to encrypt the
    files that are uploaded to the decentriq platform
    """
    material: bytes

    def __init__(self, material: Optional[bytes] = None):
        """
        Returns a new `Key` instance, can optional specify the raw key material
        """
        if material == None:
            key_bytes = os.urandom(KEY_LEN)
        else:
            if len(material) != KEY_LEN:
                raise Exception("Invalid key length, must be 32 bytes")
            key_bytes = material
        self.material = key_bytes

def create_chunk_header(extra_entropy: bytes) -> bytes:
    chunk_header = ChunkHeader()
    chunk_header.extraEntropy = extra_entropy

    chunk_header_bytes = serialize_length_delimited(chunk_header)
    return chunk_header_bytes


def create_version_header() -> bytes:
    version_header = VersionHeader()
    version_header.version = 0
    return serialize_length_delimited(version_header)

# Returns (integrity hash, encrypted blob)
def create_encrypted_chunk(
        key: bytes,
        extra_entropy: bytes,
        data: bytes
) -> Tuple[bytes, bytes]:
    chunk_bytes = []

    version_header = create_version_header()
    chunk_bytes.append(version_header)

    chunk_header = create_chunk_header(extra_entropy)
    chunk_bytes.append(chunk_header)

    chunk_bytes.append(data)

    chunk = b''.join(chunk_bytes)
    chunk_hasher = sha256()
    chunk_hasher.update(chunk)
    chunk_hash = chunk_hasher.digest()

    cipher = StorageCipher(key)
    encrypted_chunk = cipher.encrypt(chunk)

    return chunk_hash, encrypted_chunk

class Chunker(Iterator):
    def __init__(self, input_stream: BinaryIO, chunk_size: int):
        self.chunk_size = chunk_size
        self.input_stream = input_stream

    def __iter__(self) -> Iterator[Tuple[bytes, bytes]]:
        self.input_stream.seek(0)
        return self

    # returns (hash, chunk)
    def __next__(self) -> Tuple[bytes, bytes]:
        version_header_bytes = create_version_header()
        chunk_header_bytes = create_chunk_header(os.urandom(16))

        # Does not account for header size
        current_chunk_size = 0
        chunk_bytes = [version_header_bytes, chunk_header_bytes]

        input_chunk_bytes = self.input_stream.read(self.chunk_size)
        if len(input_chunk_bytes) == 0:
            raise StopIteration
        chunk_bytes.append(input_chunk_bytes)

        chunk = b''.join(chunk_bytes)
        chunk_hasher = sha256()
        chunk_hasher.update(chunk)
        chunk_hash = chunk_hasher.digest()
        return chunk_hash, chunk


class StorageCipher():
    def __init__(self, symmetric_key: bytes):
        self.enc_key = symmetric_key
        self.cipher: chily.Cipher = chily.Cipher.from_symmetric(self.enc_key)

    def encrypt(self, data: bytes) -> bytes:
        nonce = chily.Nonce.from_random()
        encrypted_data = self.cipher.encrypt(data, nonce)

        encryption_header = EncryptionHeader()
        encryption_header.chilyKey.encryptionNonce = bytes(nonce.bytes)

        serialized_encryption_header = serialize_length_delimited(encryption_header)
        encrypted_data_with_header = bytes(list(serialized_encryption_header) + encrypted_data)
        return encrypted_data_with_header



import json
from .authentication import generate_csr, generate_key, Auth
from .types import (
    UserCsrResponse,
    UserCsrRequest,
)
from .api import API, Endpoints
from .proto import (
    AuthenticationMethod, TrustedPki
)


class ClientPlatformFeatures:
    """
    Provider of a list of methods and properties mirroring what is offered by the Decentriq
    web platform.
    """
    _http_api: API

    def __init__(
        self,
        user_email: str,
        http_api: API,
    ):
        """
        Creating objects of this class directly is not necessary as an object of this class is
        provided as part of each `Client` object.
        """
        self._http_api = http_api
        self.user_email = user_email

    @property
    def decentriq_ca_root_certificate(self) -> bytes:
        """
        Returns the root certificate used by the Decentriq identity provider.
        Note that when using this certificate in any authentication scheme you trust Decentriq as an identity provider!
        """
        url = Endpoints.SYSTEM_CERTIFICATE_AUTHORITY
        response = self._http_api.get(url).json()
        return response["rootCertificate"].encode("utf-8")

    @property
    def decentriq_pki_authentication(self) -> AuthenticationMethod:
        """
        The authentication method that uses the Decentriq root certificate to authenticate
        users.

        This method should be specified when building a data room in case you want to interact
        with the that data room either via the web interface or with sessions created using
        `create_auth_using_decentriq_pki`.
        Note that when using this authentication method you trust Decentriq as an identity provider!

        You can also create an `AuthenticationMethod` object directly and supply your own root certificate,
        with which to authenticate users connecting to your data room.
        In this case you will also need to issue corresponding user certificates and create your
        own custom `Auth` objects.
        """
        root_pki = self.decentriq_ca_root_certificate
        return AuthenticationMethod(
            trustedPki=TrustedPki(rootCertificatePem=root_pki)
        )

    def create_auth_using_decentriq_pki(self, email: str = None) -> Auth:
        """
        Creates a `decentriq_platform.authentication.Auth` object which can be
        attached to a `decentriq_platform.session.Session`.
        Sessions created using such an `Auth` object will commonly be used with
        data rooms that have been configured to use the `decentriq_pki_authentication`
        authentication method.
        """
        email = email if email is not None else self.user_email
        keypair = generate_key()
        csr = generate_csr(email, keypair)
        url = Endpoints.USER_CERTIFICATE.replace(":userId", email)
        csr_req = UserCsrRequest(csrPem=csr.decode("utf-8"))
        resp: UserCsrResponse = self._http_api.post(url, req_body=json.dumps(csr_req)).json()
        cert_chain_pem = resp["certChainPem"].encode("utf-8")
        auth = Auth(cert_chain_pem, keypair, email)
        return auth



"""
..include:: ../../../decentriq_platform_docs/container_getting_started.md
---
"""
__docformat__ = "restructuredtext"

from .compute import StaticContainerCompute
from .attestation import EnclaveSpecification

__all__ = [
    "StaticContainerCompute",
    "EnclaveSpecification",
]



from .proto import CreateDataRoomResponse


def get_data_room_id(create_data_room_response: CreateDataRoomResponse) -> bytes:
    response_type = create_data_room_response.WhichOneof("create_data_room_response")
    if response_type is None:
        raise Exception("Empty CreateDataRoomResponse")
    elif response_type == "dataRoomId":
        return create_data_room_response.dataRoomId
    elif response_type == "dataRoomValidationError":
        raise Exception(
            "DataRoom creation failed",
            create_data_room_response.dataRoomValidationError
        )
    else:
        raise Exception("Unknown response type for CreateDataRoomResponse", response_type)



from .types import EnclaveSpecification
from .proto import (
    AttestationSpecification,
    AttestationSpecificationIntelEpid
)
from .verification import intel_sgx_ias_root_ca
import asn1crypto.pem


intel_sgx_ias_root_ca_der = asn1crypto.pem.unarmor(intel_sgx_ias_root_ca)[2]


class EnclaveSpecifications:
    """
    The list of available enclave specifications provided by this package.

    Enclave specifications enable you to express which particular enclaves you trust.
    The field containing the measurement (e.g. `mrenclave` in the case of Intel SGX) identifies
    the exact binary that will process your data.
    Users of the Decentriq platform are encouraged to reproduce this value by building the enclave
    binary from audited source code and re-producing the measurement (in the case of Intel SGX,
    this would involve simply hashing the produced executable).

    When connecting to the driver enclave, the configured attestation algorithm will guarantee that the
    enclave you connect to is the one corresponding to the enclave specification you chose.
    The associated root certificate will be used to verify that the attestation was signed
    by the expected party (e.g. Intel/AMD/Amazon, depending on the CC technology used).

    Any communication between the driver enclave and worker enclaves handling your data will also
    first be secured by additional attestation procedures. Which enclaves are trusted by the
    driver enclave is controlled by choosing the additional enclave specs from the respective
    compute packages.

    A list of enclave specifications, each encoding your trust in a particular enclave type, is called an
    *enclave specification set*. Enclave specification sets are obtained by concatenating multiple such specs
    and converting them to a dictionary that maps the enclave name to the trusted enclave spec.
    """
    V1_SGX = [
        EnclaveSpecification(
            name="decentriq.driver",
            version="1",
            proto= AttestationSpecification(
                intelEpid=AttestationSpecificationIntelEpid(
                    mrenclave=bytes.fromhex(
                        "ce2c7ff27a3efe21968dc1a54e662441a88f92bcf7c53b3cc00f7e579cca6591"
                    ),
                    iasRootCaDer=intel_sgx_ias_root_ca_der,
                )
            )
        )
    ]



import os

DECENTRIQ_CLIENT_ID = os.getenv("DECENTRIQ_CLIENT_ID", "MHyVW112w7Ql95G96fn9rnLWkYuOLmdk")
DECENTRIQ_HOST = os.getenv("DECENTRIQ_HOST", "api.decentriq.com")
DECENTRIQ_PORT = int(os.getenv("DECENTRIQ_PORT", "443"))
DECENTRIQ_USE_TLS = True if os.getenv("DECENTRIQ_USE_TLS", "true") == "true" else False



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class PrimitiveType(_PrimitiveType, metaclass=_PrimitiveTypeEnumTypeWrapper):
    pass
class _PrimitiveType:
    V = typing.NewType('V', builtins.int)
class _PrimitiveTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PrimitiveType.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    INT64 = PrimitiveType.V(0)
    STRING = PrimitiveType.V(1)
    FLOAT64 = PrimitiveType.V(2)

INT64 = PrimitiveType.V(0)
STRING = PrimitiveType.V(1)
FLOAT64 = PrimitiveType.V(2)
global___PrimitiveType = PrimitiveType


class SqlWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VALIDATION_FIELD_NUMBER: builtins.int
    COMPUTATION_FIELD_NUMBER: builtins.int
    @property
    def validation(self) -> global___ValidationConfiguration: ...
    @property
    def computation(self) -> global___ComputationConfiguration: ...
    def __init__(self,
        *,
        validation : typing.Optional[global___ValidationConfiguration] = ...,
        computation : typing.Optional[global___ComputationConfiguration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"computation",b"computation",u"configuration",b"configuration",u"validation",b"validation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"computation",b"computation",u"configuration",b"configuration",u"validation",b"validation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"configuration",b"configuration"]) -> typing.Optional[typing_extensions.Literal["validation","computation"]]: ...
global___SqlWorkerConfiguration = SqlWorkerConfiguration

class ValidationConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TABLESCHEMA_FIELD_NUMBER: builtins.int
    @property
    def tableSchema(self) -> global___TableSchema: ...
    def __init__(self,
        *,
        tableSchema : typing.Optional[global___TableSchema] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"tableSchema",b"tableSchema"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"tableSchema",b"tableSchema"]) -> None: ...
global___ValidationConfiguration = ValidationConfiguration

class TableSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NAMEDCOLUMNS_FIELD_NUMBER: builtins.int
    @property
    def namedColumns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NamedColumn]: ...
    def __init__(self,
        *,
        namedColumns : typing.Optional[typing.Iterable[global___NamedColumn]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"namedColumns",b"namedColumns"]) -> None: ...
global___TableSchema = TableSchema

class NamedColumn(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NAME_FIELD_NUMBER: builtins.int
    COLUMNTYPE_FIELD_NUMBER: builtins.int
    name: typing.Text = ...
    @property
    def columnType(self) -> global___ColumnType: ...
    def __init__(self,
        *,
        name : typing.Text = ...,
        columnType : typing.Optional[global___ColumnType] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"_name",b"_name",u"columnType",b"columnType",u"name",b"name"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"_name",b"_name",u"columnType",b"columnType",u"name",b"name"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_name",b"_name"]) -> typing.Optional[typing_extensions.Literal["name"]]: ...
global___NamedColumn = NamedColumn

class ColumnType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    PRIMITIVETYPE_FIELD_NUMBER: builtins.int
    NULLABLE_FIELD_NUMBER: builtins.int
    primitiveType: global___PrimitiveType.V = ...
    nullable: builtins.bool = ...
    def __init__(self,
        *,
        primitiveType : global___PrimitiveType.V = ...,
        nullable : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"nullable",b"nullable",u"primitiveType",b"primitiveType"]) -> None: ...
global___ColumnType = ColumnType

class ComputationConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    SQLSTATEMENT_FIELD_NUMBER: builtins.int
    PRIVACYSETTINGS_FIELD_NUMBER: builtins.int
    CONSTRAINTS_FIELD_NUMBER: builtins.int
    sqlStatement: typing.Text = ...
    @property
    def privacySettings(self) -> global___PrivacySettings: ...
    @property
    def constraints(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Constraint]: ...
    def __init__(self,
        *,
        sqlStatement : typing.Text = ...,
        privacySettings : typing.Optional[global___PrivacySettings] = ...,
        constraints : typing.Optional[typing.Iterable[global___Constraint]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"privacySettings",b"privacySettings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"constraints",b"constraints",u"privacySettings",b"privacySettings",u"sqlStatement",b"sqlStatement"]) -> None: ...
global___ComputationConfiguration = ComputationConfiguration

class PrivacySettings(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MINAGGREGATIONGROUPSIZE_FIELD_NUMBER: builtins.int
    minAggregationGroupSize: builtins.int = ...
    def __init__(self,
        *,
        minAggregationGroupSize : builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"minAggregationGroupSize",b"minAggregationGroupSize"]) -> None: ...
global___PrivacySettings = PrivacySettings

class Constraint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DESCRIPTION_FIELD_NUMBER: builtins.int
    description: typing.Text = ...
    def __init__(self,
        *,
        description : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"description",b"description"]) -> None: ...
global___Constraint = Constraint



"""
.. include:: ../../decentriq_platform_docs/gcg_getting_started.md
---
"""
__docformat__ = "restructuredtext"

from .types import JobId
from .client import Client, create_client, Session
from .platform import ClientPlatformFeatures
from .builders import DataRoomBuilder
from .compute import Noop, StaticContent
from .permission import Permissions
from .storage import Key
from .attestation import EnclaveSpecifications


__all__ = [
    "create_client",
    "Client",
    "Session",
    "ClientPlatformFeatures",
    "DataRoomBuilder",
    "Permissions",
    "EnclaveSpecifications",
    "Key",
    "Noop",
    "JobId",
    "sql",
    "container",
]



from ..proto.attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationAwsNitro,
)
import asn1crypto.pem
from ..certs import aws_nitro_root_ca_pem


aws_nitro_root_ca_der = asn1crypto.pem.unarmor(aws_nitro_root_ca_pem)[2]


class EnclaveSpecification:
    """
    The list of available enclave specifications provided by this compute package.

    Refer to the class with the same name located in the main package for more information.
    """
    # V0_NITRO = [
    #     {
    #         "name": "decentriq.python-worker",
    #         "version": "0",
    #         "proto": AttestationSpecification(
    #             awsNitro=AttestationSpecificationAwsNitro(
    #                 nitroRootCaDer=aws_nitro_root_ca_der,
    #                 ...
    #             )
    #         )
    #     }
    # ]



import hashlib
import json
import os
from threading import BoundedSemaphore
from concurrent import futures
from base64 import b64encode
from typing import BinaryIO, List, Dict, Optional
from .api import API, Endpoints
from .authentication import Auth
from .config import (
        DECENTRIQ_CLIENT_ID, DECENTRIQ_HOST, DECENTRIQ_PORT, DECENTRIQ_USE_TLS,
)
from .session import Session
from .storage import Key, Chunker, create_encrypted_chunk, StorageCipher
from .types import (
    EnclaveSpecification, EnclaveSpecificationResponse,
    DatasetDescription, FinalizeUpload,
    CreateSessionRequest, SessionJsonResponse,
    UploadDescription, ChunkWrapper,
    CreateScopeRequest, ScopeJson, ScopeTypes,
)
from .proto import AttestationSpecification, parse_length_delimited, serialize_length_delimited
from .platform import ClientPlatformFeatures
import base64


class Client:
    """
    A `Client` object allows you to upload datasets and to create `Session` objects that
    can communicate with enclaves and perform essential operations such as publishing
    data rooms and execute computations and retrieve results.

    Objects of this class can be used to create and run data rooms, as well as to securely
    upload data and retrieve computation results.

    Objects of this class should be created using the `create_client` function.
    this class.
    """

    _api: API
    _platform: Optional[ClientPlatformFeatures]

    def __init__(
            self,
            user_email: str,
            api: API,
            platform: Optional[ClientPlatformFeatures] = None
    ):
        """
        Create a client instance.

        Rather than creating `Client` instances directly using this constructor, use the function
        `create_client`.
        """
        self.user_email = user_email
        self._api = api
        self._platform = platform

    def create_enclave_spec_set(
            self,
            specs: List[EnclaveSpecification],
            check_availability: bool = True
    ) -> Dict[str, EnclaveSpecification]:
        """
        Create an enclave spec set from a list of `EnclaveSpecification`s, optionally checking
        whether corresponding enclaves are deployed at this moment.

        **Parameters**:
        - `specs`: A list of enclave specs obtained by concatenating the specs found in
            the compute packages.
        - `check_availability`: Whether to check if there are currently enclaves running that
            match the chosen enclave specs. If this is not the case, an exception will be thrown.

        **Returns**:
        An enclave spec set that can be used together with `create_session` or `DataRoomBuilder`.
        """
        if check_availability:
            available_specs =\
                {spec["proto"].SerializeToString() for spec in self._get_enclave_specifications()}
            for s in specs:
                if s["proto"].SerializeToString() not in available_specs:
                    raise Exception(
                        "No available enclave deployed for attestation spec '{name}' (version {version})".\
                            format(name=s["name"], version=s["version"])
                    )

        return { spec["name"]: spec for spec in specs }

    def _get_enclave_specifications(self) -> List[EnclaveSpecification]:
        url = Endpoints.SYSTEM_ATTESTATION_SPECS
        response: EnclaveSpecificationResponse = self._api.get(url).json()
        enclave_specs = []

        for spec_json in response["attestationSpecs"]:
            attestation_specification = AttestationSpecification()
            spec_length_delimited = base64.b64decode(spec_json["spec"])
            parse_length_delimited(spec_length_delimited, attestation_specification)
            enclave_spec = EnclaveSpecification(
                name=spec_json["name"],
                version=spec_json["version"],
                proto=attestation_specification
            )
            enclave_specs.append(enclave_spec)

        return enclave_specs

    def create_session(
            self,
            auth: Auth,
            enclaves: Dict[str, EnclaveSpecification],
    ) -> Session:
        """
        Creates a new `decentriq_platform.session.Session` instance to communicate
        with an enclave service with the specified identifier.

        Messages sent thorugh this session will be authenticated
        with the authentication object identifier specified during a call.
        """
        url = Endpoints.SESSIONS
        attestation_proto = enclaves["decentriq.driver"]["proto"]

        attestation_specification_hash =\
            hashlib.sha256(serialize_length_delimited(attestation_proto)).hexdigest()

        req_body = CreateSessionRequest(
            attestationSpecificationHash=attestation_specification_hash
        )
        response: SessionJsonResponse = self._api.post(
                url,
                json.dumps(req_body),
                {"Content-type": "application/json"}
        ).json()

        session = Session(
                self,
                response["sessionId"],
                attestation_proto,
                auth,
                auth.user_id
        )

        return session

    def _create_scope(self, email: str, metadata: Dict[str, str]) -> str:
        url = Endpoints.USER_SCOPES_COLLECTION.replace(":userId", email)
        req_body = CreateScopeRequest(metadata=metadata)
        response: ScopeJson = self._api.post(
                url,
                json.dumps(req_body),
                {"Content-type": "application/json"}
        ).json()
        return response["scopeId"]

    def _get_scope(self, email: str, scope_id: str) -> ScopeJson:
        url = Endpoints.USER_SCOPE \
                .replace(":userId", email) \
                .replace(":scopeId", scope_id)
        response: ScopeJson = self._api.get(url).json()
        return response

    def _get_scope_by_metadata(self, email: str, metadata: Dict[str, str]) -> Optional[str]:
        url = Endpoints.USER_SCOPES_COLLECTION.replace(":userId", email)
        response: List[ScopeJson] = self._api.get(
                url,
                params={"metadata": json.dumps(metadata)}
            ).json()
        if len(response) == 0:
            return None
        else:
            scope = response[0]
            return scope["scopeId"]

    def _ensure_scope_with_metadata(self, email: str, metadata: Dict[str, str]) -> str:
        scope = self._get_scope_by_metadata(email, metadata)
        if scope is None:
            scope = self._create_scope(email, metadata)
        return scope

    def _delete_scope(self, email: str, scope_id: str):
        url = Endpoints.USER_SCOPE \
            .replace(":userId", email) \
            .replace(":scopeId", scope_id)
        self._api.delete(url)

    def upload_dataset(
            self,
            file_input_stream: BinaryIO,
            key: Key,
            /, *,
            description: str,
            chunk_size: int = 8 * 1024 ** 2,
            parallel_uploads: int = 8,
            owner_email: Optional[str] = None,
    ) -> str:
        """
        Uploads `file_input_stream` as a file usable by enclaves and returns the
        corresponding manifest hash

        **Parameters**:
        - `owner_email`: owner of the file
        - `file_input_stream`: file content
        - `description`: file description
        - `key`: key used to encrypt the file
        """
        uploader = BoundedExecutor(
                bound=parallel_uploads * 2,
                max_workers=parallel_uploads
        )
        email = owner_email if owner_email else self.user_email
        # create and upload chunks
        chunker = Chunker(file_input_stream, chunk_size=chunk_size)
        chunk_hashes: List[str] = []
        chunk_uploads_futures = []
        upload_description = self._create_upload(email)
        for chunk_hash, chunk_data in chunker:
            chunk_uploads_futures.append(
                uploader.submit(
                    self._encrypt_and_upload_chunk,
                    chunk_hash,
                    chunk_data,
                    key.material,
                    email,
                    upload_description["uploadId"]
                )
            )
            chunk_hashes.append(chunk_hash.hex())

        # check chunks uploads were successful
        completed, pending = futures.wait(
                chunk_uploads_futures,
                None,
                futures.FIRST_EXCEPTION
            )
        if len(pending):
            # re-raise exception
            for future in completed: future.result()
        uploader.shutdown(wait=False)

        # create manifest and upload
        manifest_hash, manifest_encrypted = create_encrypted_chunk(
                key.material,
                os.urandom(16),
                json.dumps(chunk_hashes).encode("utf-8")
        )
        scope_id = self._ensure_scope_with_metadata(email, {"type": ScopeTypes.USER_FILE})
        self._finalize_upload(
            user_id=email,
            scope_id=scope_id,
            upload_id=upload_description["uploadId"],
            name=description,
            manifest_hash=manifest_hash,
            manifest_encrypted=manifest_encrypted,
            chunks=chunk_hashes
        )
        return manifest_hash.hex()

    def _encrypt_and_upload_chunk(
            self,
            chunk_hash: bytes,
            chunk_data: bytes,
            key: bytes,
            user_id: str,
            upload_id: str
    ):
        cipher = StorageCipher(key)
        chunk_data_encrypted = cipher.encrypt(chunk_data)
        self._upload_chunk(chunk_hash, chunk_data_encrypted, user_id, upload_id)

    def _create_upload(self, user_id: str) -> UploadDescription:
        url = Endpoints.USER_UPLOADS_COLLECTION.replace(":userId", user_id)
        response = self._api.post(url, {}, {"Content-type": "application/json"})
        upload_description: UploadDescription = response.json()
        return upload_description

    def _upload_chunk(
            self,
            chunk_hash: bytes,
            chunk_data_encrypted: bytes,
            user_id: str,
            upload_id: str
    ):
        url = Endpoints.USER_UPLOAD_CHUNKS \
            .replace(":userId", user_id) \
            .replace(":uploadId", upload_id)
        wrapped_chunk= ChunkWrapper(
            hash=chunk_hash.hex(),
            data=b64encode(chunk_data_encrypted).decode("ascii")
        )
        self._api.post(url, json.dumps(wrapped_chunk), {"Content-type": "application/json"})

    def _delete_user_upload(self, email: str, upload_id: str):
        url = Endpoints.USER_UPLOAD \
            .replace(":userId", email) \
            .replace(":uploadId", upload_id)
        self._api.delete(url)

    def _get_user_upload(self, email: str, upload_id: str) -> UploadDescription:
        url = Endpoints.USER_UPLOAD.replace(
                ":userId", email
            ).replace(":uploadId", upload_id)
        response = self._api.get(url)
        return response.json()

    def _get_user_uploads_collection(self, email: str) -> List[UploadDescription]:
        url = Endpoints.USER_UPLOADS_COLLECTION.replace(":userId", email)
        response = self._api.get(url)
        return response.json()

    def _finalize_upload(
            self,
            user_id: str,
            scope_id: str,
            upload_id: str,
            name: str,
            manifest_hash: bytes,
            manifest_encrypted: bytes,
            chunks: List[str]
    ) -> DatasetDescription:
        url = Endpoints.USER_FILES_COLLECTION \
            .replace(":userId", user_id) \
            .replace(":scopeId", scope_id)
        payload = FinalizeUpload(
            uploadId=upload_id,
            manifest=b64encode(manifest_encrypted).decode("ascii"),
            manifestHash=manifest_hash.hex(),
            name=name,
            chunks=chunks
        )
        dataset_description: DatasetDescription = self._api.post(
            url,
            json.dumps(payload),
            {"Content-type": "application/json"}
        ).json()
        return dataset_description

    def get_dataset(
            self,
            email: str,
            manifest_hash: str
    ) -> DatasetDescription:
        """
        Returns informations about a user file
        """
        scope_id = self._ensure_scope_with_metadata(email, {"type": ScopeTypes.USER_FILE})
        url = Endpoints.USER_FILE \
            .replace(":userId", email) \
            .replace(":scopeId", scope_id) \
            .replace(":manifestHash", manifest_hash)
        response = self._api.get(url)
        return response.json()

    def get_all_datasets(
            self,
            email: str,
    ) -> List[DatasetDescription]:
        """
        Returns the list of files uploaded by a user
        """
        scope_id = self._ensure_scope_with_metadata(email, {"type": ScopeTypes.USER_FILE})
        url = Endpoints.USER_FILES_COLLECTION \
            .replace(":userId", email) \
            .replace(":scopeId", scope_id)
        response = self._api.get(url)
        data = response.json()

        return data

    def delete_dataset(self, email: str, manifest_hash: str):
        """
        Deletes a user file from the decentriq platform
        """
        scope_id = self._ensure_scope_with_metadata(email, {"type": ScopeTypes.USER_FILE})
        url = Endpoints.USER_FILE \
            .replace(":userId", email) \
            .replace(":scopeId", scope_id) \
            .replace(":manifestHash", manifest_hash)
        self._api.delete(url)

    @property
    def platform(self) -> ClientPlatformFeatures:
        """
        Provider of a list of convenience methods to interact with the Decentriq platform.

        This field exposes an object that provides a set of features known from the Decentriq
        web platform.
        """
        if self._platform:
            return self._platform
        else:
            raise Exception(
                "This field is not set as the client has not been configured with integration"
                " with the web platform."
            )

    @property
    def is_integrated_with_platform(self) -> bool:
        """Whether this client has been created with platform integration"""
        return self._platform is not None


def create_client(
        user_email: str,
        api_token: str,
        *,
        integrate_with_platform: bool,
        client_id: str = DECENTRIQ_CLIENT_ID,
        api_core_host: str = DECENTRIQ_HOST,
        api_core_port: int = DECENTRIQ_PORT,
        api_core_use_tls: bool = DECENTRIQ_USE_TLS
) -> Client:
    """
    The primary way to create a `Client` object.

    Client objects created using this method can optionally be integrated with the
    Decentriq web platform (<http://platform.decentriq.ch>).
    This needs to be set if you want to use the authentication objects that let Decentriq
    act as the root CA that controls access to data rooms.

    **Parameters**:
    - `api_token`: An API token with which to authenticate onself.
        The API token can be obtained in the user
        panel of the decentriq platform <https://platform.decentriq.com/tokens>.
    - `user_email`: The email address of the user that generated the given API token.
    - `integrate_with_platform`: Whether to configure the client to integrate itself with our web platform.
        When this setting is set to False, none of the features provided via the
        `platform` field will be available.
    """
    api = API(
        api_token,
        client_id,
        api_core_host,
        api_core_port,
        api_prefix="/api/core",
        use_tls=api_core_use_tls
    )

    if integrate_with_platform:
        platform = ClientPlatformFeatures(
            user_email,
            http_api=api,
        )
    else:
        platform = None

    return Client(user_email, api, platform)


class BoundedExecutor:
    def __init__(self, bound, max_workers):
        self.executor = futures.ThreadPoolExecutor(max_workers=max_workers)
        self.semaphore = BoundedSemaphore(bound + max_workers)

    def submit(self, fn, *args, **kwargs):
        self.semaphore.acquire()
        try:
            future = self.executor.submit(fn, *args, **kwargs)
        except:
            self.semaphore.release()
            raise
        else:
            future.add_done_callback(lambda _: self.semaphore.release())
            return future

    def shutdown(self, wait=True):
        self.executor.shutdown(wait)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['decentriq_platform',
 'decentriq_platform.container',
 'decentriq_platform.container.proto',
 'decentriq_platform.proto',
 'decentriq_platform.sql',
 'decentriq_platform.sql.proto']

package_data = \
{'': ['*']}

install_requires = \
['asn1crypto>=1.4.0,<2.0.0',
 'certvalidator>=0.11.1,<0.12.0',
 'chily>=0.5.3,<0.6.0',
 'cryptography>=3.4.8,<4.0.0',
 'ecdsa>=0.17.0,<0.18.0',
 'oscrypto>=1.2.1,<2.0.0',
 'pem>=21.2.0,<22.0.0',
 'protobuf>=3.18.0,<4.0.0',
 'requests>=2.26.0,<3.0.0',
 'sgx-ias-structs>=0.1.7,<0.2.0',
 'sqloxide>=0.1.11,<0.2.0',
 'typing-extensions>=3.10.0,<4.0.0']

setup_kwargs = {
    'name': 'decentriq-platform',
    'version': '0.8.0.dev6',
    'description': 'Python client library for the Decentriq platform',
    'long_description': '# decentriq-platform\n\nPython client library for the Decentriq platform.\n',
    'author': 'decentriq',
    'author_email': 'opensource@decentriq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/decentriq/decentriq-platform',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from .proto import (
    DriverTaskConfig,
    NoopConfig,
    serialize_length_delimited,
    StaticContentConfig,
    ComputeNodeFormat,
)

class Noop():
    """
    Computation node which does not perform any operation and produces an empty
    output. This is mostly used to allow users to test the execution of other computation
    nodes without giving access to the results.
    """
    config: bytes
    """Serialized configuration to use in the compute node definition"""

    enclave_type: str
    """What type of enclave this node uses"""

    def __init__(self) -> None:
        self.config = serialize_length_delimited(DriverTaskConfig(noop=NoopConfig()))
        self.enclave_type = "decentriq.driver"
        self.output_format = ComputeNodeFormat.RAW


class StaticContent():
    """
    Computation node which outputs the content specified in its configuration.
    This is mostly used to allow users to specify dependencies with a static
    content, which are part of the DCR definition.
    """
    config: bytes
    """Serialized configuration to use in the compute node definition"""

    def __init__(self, content: bytes) -> None:
        self.config = serialize_length_delimited(
            DriverTaskConfig(
                staticContent=StaticContentConfig(
                    content=content
                )
            )
        )
        self.enclave_type = "decentriq.driver"
        self.output_format = ComputeNodeFormat.RAW



import requests
import socket
import platform
from requests import adapters
from enum import Enum
from urllib3.connection import HTTPConnection


class Endpoints(str, Enum):
    # System
    SYSTEM_ENCLAVE_IDENTIFIERS = "/system/enclave-identifiers",
    SYSTEM_ATTESTATION_SPECS = "/system/attestation-specs",
    SYSTEM_CERTIFICATE_AUTHORITY = "/system/certificate-authority",
    # Session
    SESSIONS = "/sessions",
    SESSION = "/session/:sessionId",
    SESSION_FATQUOTE = "/session/:sessionId/fatquote",
    SESSION_MESSAGES = "/session/:sessionId/messages",
    # User
    USERS_COLLECTION = "/users",
    USER = "/user/:userId",
    USER_UPLOADS_COLLECTION = "/user/:userId/uploads",
    USER_UPLOAD = "/user/:userId/upload/:uploadId",
    USER_UPLOAD_CHUNKS = "/user/:userId/upload/:uploadId/chunks",
    USER_CERTIFICATE = "/user/:userId/certificate",
    USER_SCOPES_COLLECTION = "/user/:userId/scopes",
    USER_SCOPE = "/user/:userId/scopes/:scopeId",
    USER_FILES_COLLECTION = "/user/:userId/scope/:scopeId/files",
    USER_FILE = "/user/:userId/scope/:scopeId/file/:manifestHash",


class APIError(Exception):
    def __init__(self, body):
        self.body = body

class AuthorizationError(APIError):
    """ """

    pass


class NotFoundError(APIError):
    """ """

    pass


class BadRequestError(APIError):
    """ """

    pass


class ServerError(APIError):
    """ """

    pass


class UnknownError(APIError):
    """ """

    pass

class HTTPAdapterWithSocketOptions(adapters.HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.socket_options = kwargs.pop("socket_options", None)
        super(HTTPAdapterWithSocketOptions, self).__init__(*args, **kwargs)

    def init_poolmanager(self, *args, **kwargs):
        if self.socket_options is not None:
            kwargs["socket_options"] = self.socket_options
        super(HTTPAdapterWithSocketOptions, self).init_poolmanager(*args, **kwargs)

class HTTPAdapterWithTCPKeepalive(HTTPAdapterWithSocketOptions):
    def __init__(self, *args, **kwargs):
        platform_socket_options = [
            (socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 60),
            (socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 5),
        ]
        if platform.system()=="Linux":
            platform_socket_options += [
                (socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 60),
            ]
        elif platform.system()=="Darwin":
            TCP_KEEPALIVE = getattr(socket, 'TCP_KEEPALIVE', 0x10)
            platform_socket_options += [
                (socket.IPPROTO_TCP, TCP_KEEPALIVE, 60)
            ]
        self.socket_options = HTTPConnection.default_socket_options + \
                [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)] + \
                platform_socket_options
        super(HTTPAdapterWithTCPKeepalive, self).__init__(*args, **kwargs)

class API:
    def __init__(
        self,
        api_token,
        client_id,
        host,
        port,
        use_tls,
        api_prefix,
        additional_auth_headers={},
    ):
        adapter = HTTPAdapterWithTCPKeepalive()
        session = requests.Session()
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        if use_tls:
            protocol = "https"
        else:
            protocol = "http"
        self.base_url = f"{protocol}://{host}:{port}{api_prefix}"
        auth_headers = {
                "Authorization": "Bearer " + api_token,
                "Authorization-Type": "app",
                "Authorization-Client": client_id
        }
        auth_headers.update(additional_auth_headers)
        session.headers.update(auth_headers)
        self.session = session

    @staticmethod
    def __check_response_status_code(response):
        if response.status_code >= 200 and response.status_code <= 204:
            pass
        elif response.status_code == 400:
            raise BadRequestError(response.content)
        elif response.status_code == 401 or response.status_code == 403:
            raise AuthorizationError(response.content)
        elif response.status_code == 404:
            raise NotFoundError(response.content)
        elif response.status_code >= 500 and response.status_code <= 504:
            raise ServerError(response.content)
        else:
            raise UnknownError(response.content)

    def post(self, endpoint, req_body=None, headers={}):
        url = self.base_url + endpoint
        response = self.session.post(url, data=req_body, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def post_multipart(self, endpoint, parts=None, headers={}):
        url = self.base_url + endpoint
        response = self.session.post(url, files=parts, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def put(self, endpoint, req_body=None, headers={}):
        url = self.base_url + endpoint
        response = self.session.put(url, data=req_body, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def patch(self, endpoint, req_body=None, headers={}):
        url = self.base_url + endpoint
        response = self.session.patch(url, data=req_body, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def get(self, endpoint, params={}, headers={}):
        url = self.base_url + endpoint
        response = self.session.get(url, params={**params}, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def head(self, endpoint, headers={}):
        url = self.base_url + endpoint
        response = self.session.head(url, headers={**headers})
        API.__check_response_status_code(response)
        return response

    def delete(self, endpoint, headers={}):
        url = self.base_url + endpoint
        response = self.session.delete(url, headers={**headers})
        API.__check_response_status_code(response)
        return response



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import attestation_pb2
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class ComputeNodeFormat(_ComputeNodeFormat, metaclass=_ComputeNodeFormatEnumTypeWrapper):
    pass
class _ComputeNodeFormat:
    V = typing.NewType('V', builtins.int)
class _ComputeNodeFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ComputeNodeFormat.V], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    RAW = ComputeNodeFormat.V(0)
    ZIP = ComputeNodeFormat.V(1)

RAW = ComputeNodeFormat.V(0)
ZIP = ComputeNodeFormat.V(1)
global___ComputeNodeFormat = ComputeNodeFormat


class DataRoom(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ID_FIELD_NUMBER: builtins.int
    COMPUTENODES_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONS_FIELD_NUMBER: builtins.int
    USERPERMISSIONS_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODS_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    OWNEREMAIL_FIELD_NUMBER: builtins.int
    id: typing.Text = ...
    @property
    def computeNodes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ComputeNode]: ...
    @property
    def attestationSpecifications(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[attestation_pb2.AttestationSpecification]: ...
    @property
    def userPermissions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UserPermission]: ...
    @property
    def authenticationMethods(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AuthenticationMethod]: ...
    name: typing.Text = ...
    description: typing.Text = ...
    ownerEmail: typing.Text = ...
    def __init__(self,
        *,
        id : typing.Text = ...,
        computeNodes : typing.Optional[typing.Iterable[global___ComputeNode]] = ...,
        attestationSpecifications : typing.Optional[typing.Iterable[attestation_pb2.AttestationSpecification]] = ...,
        userPermissions : typing.Optional[typing.Iterable[global___UserPermission]] = ...,
        authenticationMethods : typing.Optional[typing.Iterable[global___AuthenticationMethod]] = ...,
        name : typing.Text = ...,
        description : typing.Text = ...,
        ownerEmail : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attestationSpecifications",b"attestationSpecifications",u"authenticationMethods",b"authenticationMethods",u"computeNodes",b"computeNodes",u"description",b"description",u"id",b"id",u"name",b"name",u"ownerEmail",b"ownerEmail",u"userPermissions",b"userPermissions"]) -> None: ...
global___DataRoom = DataRoom

class ComputeNode(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    NODENAME_FIELD_NUMBER: builtins.int
    LEAF_FIELD_NUMBER: builtins.int
    BRANCH_FIELD_NUMBER: builtins.int
    nodeName: typing.Text = ...
    @property
    def leaf(self) -> global___ComputeNodeLeaf: ...
    @property
    def branch(self) -> global___ComputeNodeBranch: ...
    def __init__(self,
        *,
        nodeName : typing.Text = ...,
        leaf : typing.Optional[global___ComputeNodeLeaf] = ...,
        branch : typing.Optional[global___ComputeNodeBranch] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"branch",b"branch",u"leaf",b"leaf",u"node",b"node"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"branch",b"branch",u"leaf",b"leaf",u"node",b"node",u"nodeName",b"nodeName"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"node",b"node"]) -> typing.Optional[typing_extensions.Literal["leaf","branch"]]: ...
global___ComputeNode = ComputeNode

class ComputeNodeLeaf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ISREQUIRED_FIELD_NUMBER: builtins.int
    isRequired: builtins.bool = ...
    def __init__(self,
        *,
        isRequired : builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"isRequired",b"isRequired"]) -> None: ...
global___ComputeNodeLeaf = ComputeNodeLeaf

class ComputeNodeBranch(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    CONFIG_FIELD_NUMBER: builtins.int
    DEPENDENCIES_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONINDEX_FIELD_NUMBER: builtins.int
    OUTPUTFORMAT_FIELD_NUMBER: builtins.int
    config: builtins.bytes = ...
    @property
    def dependencies(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    attestationSpecificationIndex: builtins.int = ...
    outputFormat: global___ComputeNodeFormat.V = ...
    def __init__(self,
        *,
        config : builtins.bytes = ...,
        dependencies : typing.Optional[typing.Iterable[typing.Text]] = ...,
        attestationSpecificationIndex : builtins.int = ...,
        outputFormat : global___ComputeNodeFormat.V = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"attestationSpecificationIndex",b"attestationSpecificationIndex",u"config",b"config",u"dependencies",b"dependencies",u"outputFormat",b"outputFormat"]) -> None: ...
global___ComputeNodeBranch = ComputeNodeBranch

class UserPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EMAIL_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODINDEX_FIELD_NUMBER: builtins.int
    PERMISSIONS_FIELD_NUMBER: builtins.int
    email: typing.Text = ...
    authenticationMethodIndex: builtins.int = ...
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Permission]: ...
    def __init__(self,
        *,
        email : typing.Text = ...,
        authenticationMethodIndex : builtins.int = ...,
        permissions : typing.Optional[typing.Iterable[global___Permission]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"authenticationMethodIndex",b"authenticationMethodIndex",u"email",b"email",u"permissions",b"permissions"]) -> None: ...
global___UserPermission = UserPermission

class AuthenticationMethod(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    TRUSTEDPKI_FIELD_NUMBER: builtins.int
    @property
    def trustedPki(self) -> global___TrustedPki: ...
    def __init__(self,
        *,
        trustedPki : typing.Optional[global___TrustedPki] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"_trustedPki",b"_trustedPki",u"trustedPki",b"trustedPki"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"_trustedPki",b"_trustedPki",u"trustedPki",b"trustedPki"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"_trustedPki",b"_trustedPki"]) -> typing.Optional[typing_extensions.Literal["trustedPki"]]: ...
global___AuthenticationMethod = AuthenticationMethod

class TrustedPki(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    ROOTCERTIFICATEPEM_FIELD_NUMBER: builtins.int
    rootCertificatePem: builtins.bytes = ...
    def __init__(self,
        *,
        rootCertificatePem : builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"rootCertificatePem",b"rootCertificatePem"]) -> None: ...
global___TrustedPki = TrustedPki

class Permission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    EXECUTECOMPUTEPERMISSION_FIELD_NUMBER: builtins.int
    LEAFCRUDPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEAUDITLOGPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    UPDATEDATAROOMSTATUSPERMISSION_FIELD_NUMBER: builtins.int
    RETRIEVEPUBLISHEDDATASETSPERMISSION_FIELD_NUMBER: builtins.int
    DRYRUNPERMISSION_FIELD_NUMBER: builtins.int
    @property
    def executeComputePermission(self) -> global___ExecuteComputePermission: ...
    @property
    def leafCrudPermission(self) -> global___LeafCrudPermission: ...
    @property
    def retrieveDataRoomPermission(self) -> global___RetrieveDataRoomPermission: ...
    @property
    def retrieveAuditLogPermission(self) -> global___RetrieveAuditLogPermission: ...
    @property
    def retrieveDataRoomStatusPermission(self) -> global___RetrieveDataRoomStatusPermission: ...
    @property
    def updateDataRoomStatusPermission(self) -> global___UpdateDataRoomStatusPermission: ...
    @property
    def retrievePublishedDatasetsPermission(self) -> global___RetrievePublishedDatasetsPermission: ...
    @property
    def dryRunPermission(self) -> global___DryRunPermission: ...
    def __init__(self,
        *,
        executeComputePermission : typing.Optional[global___ExecuteComputePermission] = ...,
        leafCrudPermission : typing.Optional[global___LeafCrudPermission] = ...,
        retrieveDataRoomPermission : typing.Optional[global___RetrieveDataRoomPermission] = ...,
        retrieveAuditLogPermission : typing.Optional[global___RetrieveAuditLogPermission] = ...,
        retrieveDataRoomStatusPermission : typing.Optional[global___RetrieveDataRoomStatusPermission] = ...,
        updateDataRoomStatusPermission : typing.Optional[global___UpdateDataRoomStatusPermission] = ...,
        retrievePublishedDatasetsPermission : typing.Optional[global___RetrievePublishedDatasetsPermission] = ...,
        dryRunPermission : typing.Optional[global___DryRunPermission] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"dryRunPermission",b"dryRunPermission",u"executeComputePermission",b"executeComputePermission",u"leafCrudPermission",b"leafCrudPermission",u"permission",b"permission",u"retrieveAuditLogPermission",b"retrieveAuditLogPermission",u"retrieveDataRoomPermission",b"retrieveDataRoomPermission",u"retrieveDataRoomStatusPermission",b"retrieveDataRoomStatusPermission",u"retrievePublishedDatasetsPermission",b"retrievePublishedDatasetsPermission",u"updateDataRoomStatusPermission",b"updateDataRoomStatusPermission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"dryRunPermission",b"dryRunPermission",u"executeComputePermission",b"executeComputePermission",u"leafCrudPermission",b"leafCrudPermission",u"permission",b"permission",u"retrieveAuditLogPermission",b"retrieveAuditLogPermission",u"retrieveDataRoomPermission",b"retrieveDataRoomPermission",u"retrieveDataRoomStatusPermission",b"retrieveDataRoomStatusPermission",u"retrievePublishedDatasetsPermission",b"retrievePublishedDatasetsPermission",u"updateDataRoomStatusPermission",b"updateDataRoomStatusPermission"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"permission",b"permission"]) -> typing.Optional[typing_extensions.Literal["executeComputePermission","leafCrudPermission","retrieveDataRoomPermission","retrieveAuditLogPermission","retrieveDataRoomStatusPermission","updateDataRoomStatusPermission","retrievePublishedDatasetsPermission","dryRunPermission"]]: ...
global___Permission = Permission

class ExecuteComputePermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    COMPUTENODENAME_FIELD_NUMBER: builtins.int
    computeNodeName: typing.Text = ...
    def __init__(self,
        *,
        computeNodeName : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"computeNodeName",b"computeNodeName"]) -> None: ...
global___ExecuteComputePermission = ExecuteComputePermission

class LeafCrudPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    LEAFNODENAME_FIELD_NUMBER: builtins.int
    leafNodeName: typing.Text = ...
    def __init__(self,
        *,
        leafNodeName : typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"leafNodeName",b"leafNodeName"]) -> None: ...
global___LeafCrudPermission = LeafCrudPermission

class RetrieveDataRoomPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveDataRoomPermission = RetrieveDataRoomPermission

class RetrieveAuditLogPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveAuditLogPermission = RetrieveAuditLogPermission

class RetrieveDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrieveDataRoomStatusPermission = RetrieveDataRoomStatusPermission

class UpdateDataRoomStatusPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___UpdateDataRoomStatusPermission = UpdateDataRoomStatusPermission

class RetrievePublishedDatasetsPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___RetrievePublishedDatasetsPermission = RetrievePublishedDatasetsPermission

class DryRunPermission(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    def __init__(self,
        ) -> None: ...
global___DryRunPermission = DryRunPermission
